// ********************************************************************
// VNC1L-1A USB-Hostcontroller, USB-Stick am Mikrokontroller 
// LCD DIP204-4 4x20 (bei Bedarf)
// Version 1.1 / Stand: 06.08.2009
// History: V1.0 05.03.2009 Erstversion
//          V1.1 06.08.2009 Append-Problem bei vnc_wr_txtfile behoben 
// SDCC-Version: M-IDE-51/SDCC 2.8.0
// AT89C51ED2: --code-loc 0x0000 --iram-size 256 --xram-loc 0x0000 --xram-size 0x0700
// AT89S8253:  --code-loc 0x0000 --iram-size 256
// ********************************************************************
// Code-Speicherbedarf: ca. 5,4kByte (inkl. LCD-Funktionen)
//                      ca. 3,9kByte (ohne LCD-Funktionen)
// IRAM-Speicherbedarf: ca. 160 Byte + Stack
// XRAM-Speicherbedarf: -
// Frequenz: 11.0592 MHz
// Das Programm verwendet nur 8051/52 Standard-SFRs und wurde mit dem
// Atmel AT89C51ED2 im X1 und X2 Mode getestet.
// Der USB-Stick muss mit FAT oder FAT32 formatiert sein. Es wird nur
// eine primre Partion untersttzt. NTFS Systeme funktionieren nicht.
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich (Kontakt: az51@gmx.net).
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************

#include "at89c51ed2.h"   // Register des AT89C51ED2
#include <string.h>       // Stingfunktionen
#include <stdio.h>        // Standardfunktionen

// Typendefinition
typedef unsigned char byte;
typedef unsigned int  word;
typedef unsigned long dword;

// Definebereich
#define SCLK        P1_6
#define SDI         P1_7
#define SDO         P1_5
#define CS_VNC      P1_0
#define RE_VNC      P1_1
#define LED_D1      P0_7
#define PROMPT_CHECK           2
#define PROMPT_UND_ERROR_CHECK 1
#define PROMPT_OK              1
#define FILE_NEW               0
#define FILE_APPEND            1

// Variablenbereich
static byte batch = 0;       // Batchprozess in Main-Funktion
struct vnc_struc {
  byte rec_buffer[10];       // Empfangsbuffer letzte Daten
  byte send_buffer[20];      // Sendebuffer
  byte status;               // Status Bit fr die Empfangs-Daten
};
struct vnc_struc idata vnc;  // VNC Datensatz 

byte code *code file1[] = { // Inhalt Test-File 1 (Text)
/*  0 */ "Systemkonfiguration\x00D\x00A",
/*  1 */ "V1.1 vom 06.08.2009\x00D\x00A",
/*  2 */ "Device: AT89C51ED2\x00D\x00A",
/*  3 */ "Freq.: 11.0592MHz\x00D\x00A",
/*  4 */ "Hardware: PRO-51\x00D\x00A",
/*  5 */ "USB Hst: VDIP1 VNC1L\x00D\x00A",
/*  6 */ "Firmware: 03.68 VDAP\x00D\x00A",
/*  7 */ "Anzeige: DIP204 4x20\x00D\x00A",
/*  8 */ "Compiler: SDCC 2.9.0\x00D\x00A",
/*  9 */ "Prg.: FLIP 3.3.2\x00D\x00A",
/* 10 */ "\x0FF"}; // File-Endekennung

byte code file2[] = { // Inhalt Test-File 2 (Binr)
  0xFA,0x89,0x11,0x23,0xAA,0x02,0x00,0x1F,
  0x1D,0xBA,0xA6,0x24,0x99,0x2D,0x1E,0x1F,
  0x0D,0x0C,0xA0,0x55,0x30,0x1F,0x16,0xEF,
  0x04,0x22,0x00,0x00,0xCF,0x12,0x5F,0xDF,
  0x69,0x00,0x00,0x1F,0xEF,0x15,0x3F,0xCF,
  0x5F,0x44,0x1F,0x1F,0xBF,0x16,0x13,0x7F,
  0x00,0x10,0x1F,0xDF,0x1F,0x8F,0x16,0xDF,
  0x1F,0x32,0x3F,0x56,0x1F,0x19,0x17,0xFF,
  0x3F,0x42,0xFE,0x46,0x6F,0x99,0x10,0xAF,
  0x6F,0x62,0x36,0x66,0x10,0xAA,0xBB,0xCC};

byte idata test_buffer[25]; // Testbuffer fr eingelesenes File, Lnge wird nicht geprft!!!

#include "delay.c"     // Pausen-Funktionen

#define LCD_MONITORING
#ifdef  LCD_MONITORING // LCD-Funktionen bei Bedarf
  #define SPALTEN_MAX 20
  #define ZEILEN_MAX   4
  #define SCOLL_DELAY 600
  #define LCD_E1      P3_5
  #define LCD_RS      P3_7
  #define LCD_RW      P3_6
  #define LCD_DATA    P2
  struct lcd_struc {
    byte x;  // Spalte 
    byte y;  // Zeile
  };
  struct lcd_struc idata lcd;  // LCD Datensatz 
  #include "dip204.c"          // LCD-Basis-Funktionen fr DIP204-4 KS0073
#endif

// ********************************************************************
// Grund-Initialisierung
// ********************************************************************
void init_system (void) {
  LED_D1  = 1;    // LED D1 ausschalten
}

// ***********************************************************************
// VNC1L - USB HOST CONTROLLER - SPI-MODE (MC: Master, VNC1L: SLAVE)
// VDIP1: Jumper-Set: J3 - Pulldown / J4 - Pullup
//        VDAP Firmware muss installiert sein, mit Version 3.66 getestet
// ***********************************************************************

// ***********************************************************************
// VNC1L - Intialisierung SPI-Interface und VNC1L
// SPI Mode
// ***********************************************************************
void vnc_init(void) {
  CS_VNC = 0;    // SPI-Interface inaktiv
  SCLK   = 0;    // Ruhezustand der SPI-Leitungen herstellen
  SDI    = 1;
  SDO    = 0;
  RE_VNC = 0;    // Hardware-Reset VNC1L
  delay_ms(100); // Pause
  RE_VNC = 1;    // VNC1L starten
}

// ***********************************************************************
// VNC1L - Senden eines Zeichens
// SPI Mode
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: statusbit : 0=Accept (ok) 1=Reject (nicht ok)
// ***********************************************************************
byte vnc_wr_byte(byte wr_byte) {
byte stelle, status;
  CS_VNC = 1;           // SPI-Interface aktiv
  SDI    = 1;           // Startbit (immer 1)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  SDI    = 0;           // RW-Bit (0=Write)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  SDI    = 0;           // ADDR (0=Daten)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  // 8 Datenbits senden (MSB zuerst)
  for ( stelle = 0x80; stelle; stelle >>= 1 ) {
    SDI  = wr_byte & stelle;
    SCLK = 1; SCLK = 0; // L/H Flanke
  }
  // Statusbit einlesen
  status = SDO;         // Statusbit (0=Accept, 1=Reject)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  // End Sequenz
  CS_VNC = 0;           // SPI-Interface inaktiv
  SCLK   = 1; SCLK = 0; // L/H Flanke
  SDO    = 1;           // Datenlatch freigeben (8051-Spezifik)
#ifdef LCD_MONITORING
  lcd_monitor_byte(wr_byte); // gesendetes Zeichen auf LCD anzeigen
#endif
  return status;
} 

// ***********************************************************************
// VNC1L - Empfangen eines Zeichens
// SPI Mode
// bergabe: rd_fkt    : Funktion (0=Datenbyte, 1=Statusbyte)
// Rckgabe: Datenbyte/Statusbit ber vnc-Stukturvariable
// ***********************************************************************
void vnc_rd_byte(byte rd_fkt) {
byte stelle, daten;
  daten  = 0;
  CS_VNC = 1;           // SPI-Interface aktiv
  SDI    = 1;           // Startbit (immer 1)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  SDI    = 1;           // RW-Bit (1=Read)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  if (rd_fkt) SDI  = 1; else SDI = 0; // ADDR (0=Daten, 1=SPI-Statusinformation)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  // 8 Datenbits empfangen (MSB zuerst)
  for ( stelle = 0x80; stelle; stelle >>= 1 ) {
    if (SDO) daten |= stelle;
    SCLK = 1; SCLK = 0; // L/H Flanke
  }
  // Statusbit einlesen
  vnc.status = SDO;     // Statusbit (0=New Data, 1=Old Data)
  SCLK   = 1; SCLK = 0; // L/H Flanke
  // End Sequenz
  CS_VNC = 0;           // SPI-Interface inaktiv
  SCLK   = 1; SCLK = 0; // L/H Flanke
  SDO    = 1;           // Datenlatch freigeben (8051-Spezifik)
  if (!vnc.status) {
    // empfangenes Zeichen ist ein neues Zeichen
    // Empfangsbuffer um eine Stelle verschieben
    for ( stelle = 9; stelle; stelle-- ) vnc.rec_buffer[stelle] = vnc.rec_buffer[stelle - 1];
    vnc.rec_buffer[0] = daten; // neues Zeichen an 1. Stelle speichern
#ifdef LCD_MONITORING
    lcd_monitor_byte(vnc.rec_buffer[0]); // empfangenes Zeichen auf LCD anzeigen
#endif
  }
} 

// ***********************************************************************
// VNC1L - Senden eines Kommandos
// bergabe: wr_cmd : Kommando (Adresse)
// ***********************************************************************
void vnc_wr_cmd(byte *wr_cmd) {
byte vnc_i; // vnc_offset;
  for(vnc_i = strlen(wr_cmd); vnc_i; vnc_i--) {
    if (vnc_wr_byte(*wr_cmd)) LED_D1 = 0;  // LED_D1 nur zum Testen, EIN falls berlauf!!!
    wr_cmd++;
  }
  if (vnc_wr_byte(0x0D)) LED_D1 = 0; // Endkennung
  delay_ms(200); // Pause, um VNC1L Zeit zur Antwort zu geben
}

// ***********************************************************************
// VNC1L - Auf Antwortstring warten und analysieren
// bergabe: 1 = Prfung auf Prompt und Fehlermeldungen
//           2 = Prfung nur auf Prompt
// Rckgabe: 1 = Prompt empfangen
//           2 = command failed empfangen
// ***********************************************************************
byte vnc_rd_answer(byte buffer_check) {
byte antw = 0;
  while(1) {
    vnc_rd_byte(0); // Ein Zeichen per SPI von VNC1L empfangen
    switch (buffer_check) {
        case 1: // command failed
                if ((vnc.rec_buffer[2] == 'l') && (vnc.rec_buffer[1] == 'e') && (vnc.rec_buffer[0] == 'd')) antw = 2;
                // hier knnte auf weitere Fehlermeldungen gecheckt werden...
                // kein break; hier!!!
        case 2: // nur Prompt Check
                if ((vnc.rec_buffer[2] == 0x5C) && (vnc.rec_buffer[1] == '>') && (vnc.rec_buffer[0] == 0x0D)) {
                  // Test auf \> und CR war ok
                  antw = 1;
                  vnc.rec_buffer[0] = 0; // Buffer verwerfen (letzes Zeichen lschen reicht aus!)
                }
    }
    if (antw == 2) vnc_wr_byte(0x0D); // zustzliches CR um Fehler zu besttigen, dann kommt wieder ein Prompt (???)
    if (antw) break;
  }
  return antw;
}

// ***********************************************************************
// VNC1L - Erzeugen eines Binr-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Datenblockes
//           filelen    : Anzahl der Bytes
// ***********************************************************************
void vnc_wr_binfile(byte *filename, byte *fileinhalt, word filelen) {
word vnc_i;
byte batch_wr = 0;
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               vnc_wr_cmd("SEK 0");
               break;
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %d",filelen);
               vnc_wr_cmd(vnc.send_buffer);
               for(vnc_i = 0; vnc_i < filelen; vnc_i++) {
                 vnc_wr_byte(*fileinhalt);
                 fileinhalt++;
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
    }
    delay_ms(500); // Pause
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
}

// ***********************************************************************
// VNC1L - Erzeugen eines Text-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Textblockes
//           funktion   : File 0=Neu oder 1=Inhalt anhngen
// ***********************************************************************
void vnc_wr_txtfile(byte *filename, byte *fileinhalt, byte funktion) {
word vnc_i;
word filelen, filecr;
byte *fileptr;
byte batch_wr = 0;
  // Byte-Anzahl berechnen
  fileptr = fileinhalt;
  filelen = 0;
  filecr  = 0;
  while (*fileptr != 0xFF) {
    if (*fileptr) filelen++; else filecr++;
    fileptr++;
  }
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
      case 1:  // File Position bei Bedarf auf Anfang setzen
               if (!funktion) {
                 vnc_wr_cmd("SEK 0");
                 break;
               } else batch_wr++;  // Nun wird sofort auch noch case 2: ausgefhrt
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %d",filelen);
               vnc_wr_cmd(vnc.send_buffer);
               fileptr = fileinhalt;
               for(vnc_i = 0; vnc_i < (filelen + filecr); vnc_i++) {
                 if (*fileptr) vnc_wr_byte(*fileptr); // NULL-Terminierung ausblenden
                 fileptr++;
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
    } // Switch
    delay_ms(500); // Pause
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
}

// ***********************************************************************
// VNC1L - Einlesen eines Files vom dem USB-Stick in den Speicher
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Ziel-Datenspeicheres
//           filelen    : Anzahl der Bytes
// ***********************************************************************
void vnc_rd_file(byte *filename, byte *fileinhalt, word filelen) {
byte batch_wr = 0;
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File fr lesen ffnen
               sprintf (&vnc.send_buffer[0],"OPR %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               vnc_wr_cmd("SEK 0");
               break;
      case 2:  // File Byteanzahl einlesen
               sprintf (&vnc.send_buffer[0],"RDF %d",filelen);
               vnc_wr_cmd(vnc.send_buffer);
               while (filelen) {
                 vnc_rd_byte(0);
                 if (!vnc.status) {
                   // neues Zeichen eingelesen (bisher keine Prfung auf Bufferberlauf!!!)
                   *fileinhalt = vnc.rec_buffer[0];
                   fileinhalt++;
                   filelen--;
                 }
               }
               // Die gelesenen Zeichen stehten nun im Buffer und knnen ausgewertet werden.
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               vnc_wr_cmd(vnc.send_buffer);
               break;
    }
    delay_ms(500); // Pause
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
}


// ********************************************************************
// Hauptprogramm
// ********************************************************************
void main (void) {
  init_system();  // Hardware-Initialisierungen
  vnc_init();     // VNC1L-Initialisierung
#ifdef LCD_MONITORING
  init_lcd();     // LCD-Initialisierung 
  printf_lcd(0,0,"VNC1L-1A -Test-",1); // Einschaltmeldung  
  printf_lcd(0,1,"DIP204 (KS0073)",1);  
  printf_lcd(0,2,"AT89C51ED2",1);  
  printf_lcd(0,3,"V1.1 06.08.2009 SDCC",1);  
  delay_ms(2000);  // Pause
  clear_lcd();     // LCD lschen
#endif
  // Hauptschleife
  while(1) {
    if (vnc_rd_answer(PROMPT_CHECK) == PROMPT_OK) { // Prfung auf Promptzeichen >
      // VNC1L hat Promptzeichen > gesendet und kann jetzt neue Kommandos annehmen
      switch (batch) { // Batchprozess
        case 0: vnc_wr_cmd("ECS"); // Extented Command Set
                break;
        case 1: vnc_wr_cmd("IPA"); // Monitor Commands Value use ASCII
                break;
        case 2: vnc_wr_cmd("FWV"); // FWV Firmwareversion anzeigen
                break;
        case 3: vnc_wr_cmd("DIR"); // Dir Command
                break;
        case 4: vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_NEW);    // Textfile auf USB-Stick erzeugen
                break;
        case 5: vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_APPEND); // gleiches Textfile auf USB-Stick anhngen
                break;
        case 6: vnc_wr_binfile("VNCFILE2.BIN",file2,80);             // Binrfile auf USB-Stick erzeugen
                break;
        case 7: vnc_rd_file("VNCFILE1.TXT",test_buffer,20);          // File (Teil) von USB-Stick in Speicher einlesen
                break;
      }
      delay_ms(1000); // Pause
      batch++;        // nchster Schritt
      if (batch == 8) batch = 3; // Funktionen wiederholen fr Testzwecke
    }
  }
}
